% Analyzes the short- and long-run effects of a change in a structural parameter. 
% The program is written to consider changes in the output elasticity of capital, 
% but changes in any other parameter can be considered alternatively.
% This can be changed right after the block computing steady-state levels for initial
% structural parameter values.
%
% The single Figure displays the transition paths from the initial steady-state 
% levels for the endogenous variables to the steady-state levels after the 
% structural change.
%
% This program calls function ss_ck_c.m, which 
% lays out first order conditions for the planner's problem and computes 
% the transition matrix in the log-linear approximation to the model.
% Numerical values for structural parameters, the initial stock of capital, 
% and the number of periods can be changed in the first panel.
%
% The structural parameter to be changed and its new value can be changed where indicated.
%
% [Copyright: J.Ruiz]

clear

% Structural parameters (benchmark values)

theta=0.04;     % time discount factor
alpha=0.30;     % output elasticity of capital
n=0;            % population growth
delta=0.1;      % depreciation rate
sigma=2;        % relative risk aversion
eps=1/sigma;    % elasticity of substitution of consumption
alphab = alpha; 
sigmab = sigma;

nobs=100;   	 % periods


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% steady state before the change

kssb=(alpha/(n+delta+theta))^(1/(1-alpha));  % stock of capital
yssb=kssb^alpha;                              % output
cssb=yssb-(delta+n)*kssb;                      % consumption
issb=(delta+n)*kssb;                          % investment
ussb=(cssb^(1-sigma)-1)/(1-sigma);            % single period utility
riyb=issb/yssb;                                % invesment to output ratio
rcyb=cssb/yssb;                                % consumption to output ratio

k0=kssb;    % Initial capital stock at the steady-state before the structural change

% Change to be considered in structural parameter.
% Simultaneous changes in more than one poarameter can also be analyzed by 
% uncommenting more than on line.
 alpha= 1.05*alpha;
% sigma = 1.05*sigma;
% n = 1.05*n;
% delta = 1.05*delta
% theta = 1.05*theta;

% steady state after the change
kss=(alpha/(n+delta+theta))^(1/(1-alpha));  % stock of capital
yss=kss^alpha;                              % output
css=yss-(delta+n)*kss;                      % consumption
iss=(delta+n)*kss;                          % investment
uss=(css^(1-sigma)-1)/(1-sigma);            % single period utility
riy=iss/yss;                                % invesment to output ratio
rcy=css/yss;                                % consumption to output ratio

% Transition path
param=[theta alpha n delta sigma];
x0=[log(kss) log(css)]';
J=jacobian('ss_ck_c',x0,param);

[Mvec,Meig]=eig(J);
Meig2=diag(Meig);
[Meig3,Mord]=sort(Meig2);

  if Meig3(1)>0
     'error - no solution'
  end

  if Meig3(2)<0
     'error - multiple solutions'
  end
Mlambda=diag(Meig3);
MM=Mvec(:,Mord);
Mm=inv(MM);      
kestab=-Mm(2,1)/Mm(2,2);  % stabilization constant

vlk=zeros(nobs,1); vlk(1:11)=log(k0);
vlc=zeros(nobs,1); vlc(1:10)=log(cssb); % before the change
                   vlc(11)=log(css)+kestab*(log(k0)-log(kss)); %after the change
vk=zeros(nobs,1); vk(1:11)=exp(vlk(1:11));
vc=zeros(nobs,1); vc(1:11)=exp(vlc(1:11)); c0=vc(11);
vy=zeros(nobs,1); vy(1:11)=vk(1:11).^alphab; %before the change
vi=zeros(nobs,1); vi(1:11)=vy(1:11)-vc(1:11);
vu=zeros(nobs,1); vu(1:11)=(vc(1:11).^(1-sigmab)-1)./(1-sigmab);
for i=1:nobs-11
  lk=log(kss)+(log(k0)-log(kss))*exp(Mlambda(1,1)*i);
  lc=log(css)+(log(c0)-log(css))*exp(Mlambda(1,1)*i);
  vlk(i+11)=lk;
  vlc(i+11)=lc;
  vk(i+11)=exp(lk);
  vc(i+11)=exp(lc);
  vy(i+11)=vk(i+11)^alpha;
  vi(i+11)=vy(i+11)-vc(i+11);
  vu(i+11)=(vc(i+11)^(1-sigma)-1)/(1-sigma);
end

    vkss=ones(nobs,1).*kss;
    vcss=ones(nobs,1).*css;
    vyss=ones(nobs,1).*yss;
    viss=ones(nobs,1).*iss;
    vuss=ones(nobs,1).*uss;

figure;
hold on

subplot(3,2,1)
plot([vk,vkss]);
title('capital stock');
xlabel('periods');

subplot(3,2,2)
plot([vc,vcss]);
title('consumption');
xlabel('periods');

subplot(3,2,3)
plot([vy,vyss]);
title('output');
xlabel('periods');

subplot(3,2,4)
plot([vi,viss]);
title('investment');
xlabel('periods');

subplot(3,2,5)
plot([vu,vuss]);
title('single-period utility');
xlabel('periods');

hold off