% This program analyzes the sensitivity of steady state levels of the endogenous variables 
% in the continuous time version of the Cass Koopmans model with respect to values of 
% structural parameters. The results can be interpreted as steady-state (long-run) effects 
% of a permanent change in a structural parameter.
%
% Numerical values for structural parameters can be changed in the first panel.
%
% Figure 1 displays changes in steady state levels as the time discount factor changes
% Figure 2 shows cteady-state changes when there is a change in the output elasticity of
% capital. Figure 3 presents changes when population growth changes. Figure 4 shows the
% effects of a change in depreciation, and Figure 5 considers a change in risk aversion.
%
% [Copyright: J.Ruiz]

clear

% Structural parameters (benchmark values)

theta=0.04;     % time discount factor
alpha=0.30;     % output elasticity of capital
n=0;            % population growth
delta=0.1;      % depreciation rate
sigma=2;        % relative risk aversion
eps=1/sigma;    % elasticity of substitution of consumption

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% steady state computations

kss=(alpha/(n+delta+theta))^(1/(1-alpha));  % stock of capital
yss=kss^alpha;                              % output
css=yss-(delta+n)*kss;                      % consumption
iss=(delta+n)*kss;                          % investment
uss=(css^(1-sigma)-1)/(1-sigma);            % single period utility
riy=iss/yss;                                % invesment to output ratio
rcy=css/yss;                                % consumption to output ratio

%long run effects of a permanent change in a structural parameter.

param=[theta alpha n delta sigma];
lx0=[log(kss) log(css)]';
    param1=param;
    lx01=lx0;
    
    % change in time discount
    Mtheta=zeros(21,10);
    for i=1:21
        i
        param1(1)=(i-1)/100;
        Mtheta(i,1)=param1(1);
        lx=fsolve('ss_ck_c',lx01,optimset,param1);
        J=jacobian('ss_ck_c',lx,param1);
        [Mvec,Meig]=eig(J);
        Meig2=diag(Meig);
        [Meig3,Mord]=sort(Meig2);
        Mlambda=diag(Meig3);
        MM=Mvec(:,Mord);
        Mm=inv(MM);      
        kestab=-Mm(2,1)/Mm(2,2);  % stabilization constant
        
        lx01=lx;
        
        Mtheta(i,2)=exp(lx(1));                 % stock of capital
        Mtheta(i,3)=Mtheta(i,2)^alpha;          % otuput
        Mtheta(i,4)=exp(lx(2));                 % consumption
        Mtheta(i,5)=(n+delta)*Mtheta(i,2);      % investment
        Mtheta(i,6)=Mtheta(i,4)/Mtheta(i,3);    % consumption/output
        Mtheta(i,7)=Mtheta(i,5)/Mtheta(i,3);    % investment/output
        Mtheta(i,8)=(Mtheta(i,4)^(1-sigma)-1)/(1-sigma); % utility
        Mtheta(i,9)=abs(Mlambda(1,1))*100;      % speed of convergence
        Mtheta(i,10)=log(0.5)/Mlambda(1,1);     % time to cover half the distance to steady-state
    end
    param1=param;
    lx01=lx0;
    
    % change in output elasticity of capital
    Malpha=zeros(21,10);
    for i=1:21
        param1(2)=0.2+(i-1)/100;
        Malpha(i,1)=param1(2);
        lx=fsolve('ss_ck_c',lx01,optimset,param1);
        J=jacobian('ss_ck_c',lx,param1);
        [Mvec,Meig]=eig(J);
        Meig2=diag(Meig);
        [Meig3,Mord]=sort(Meig2);
        Mlambda=diag(Meig3);
        MM=Mvec(:,Mord);
        Mm=inv(MM);      
        kestab=-Mm(2,1)/Mm(2,2);  % stabilization constant
        
        lx01=lx;
        Malpha(i,2)=exp(lx(1));                 % stock of capital
        Malpha(i,3)=Malpha(i,2)^Malpha(i,1);    % otuput
        Malpha(i,4)=exp(lx(2));                 % consumption
        Malpha(i,5)=(n+delta)*Malpha(i,2);      % investment
        Malpha(i,6)=Malpha(i,4)/Malpha(i,3);    % consumption/output
        Malpha(i,7)=Malpha(i,5)/Malpha(i,3);    % investment/output
        Malpha(i,8)=(Malpha(i,4)^(1-sigma)-1)/(1-sigma); % utility
        Malpha(i,9)=abs(Mlambda(1,1))*100;      % speed of convergence
        Malpha(i,10)=log(0.5)/Mlambda(1,1); 		% time to cover half the distance to steady-state
    end
    param1=param;
    lx01=lx0;
    
    % change in population growth
    Mn=zeros(21,10);
    for i=1:21
        param1(3)=-0.05+(i-1)/100;
        Mn(i,1)=param1(3);
        lx=fsolve('ss_ck_c',lx01,optimset,param1);
        J=jacobian('ss_ck_c',lx,param1);
        [Mvec,Meig]=eig(J);
        Meig2=diag(Meig);
        [Meig3,Mord]=sort(Meig2);
        Mlambda=diag(Meig3);
        MM=Mvec(:,Mord);
        Mm=inv(MM);      
        kestab=-Mm(2,1)/Mm(2,2);  % stabilization constant
        
        lx01=lx;
        Mn(i,2)=exp(lx(1));                 	% stock of capital
        Mn(i,3)=Mn(i,2)^alpha;          		% otuput
        Mn(i,4)=exp(lx(2));                 	% consumption
        Mn(i,5)=(Mn(i,1)+delta)*Mn(i,2);     % investment
        Mn(i,6)=Mn(i,4)/Mn(i,3);    			% consumption/output
        Mn(i,7)=Mn(i,5)/Mn(i,3);    			% investment/output
        Mn(i,8)=(Mn(i,4)^(1-sigma)-1)/(1-sigma); % utility
        Mn(i,9)=abs(Mlambda(1,1))*100;      	% speed of convergence
        Mn(i,10)=log(0.5)/Mlambda(1,1); 		% time to cover half the distance to steady-state
    end
    param1=param;
    lx01=lx0;
    % change in depreciation rate
    Mdelta=zeros(21,10);
    for i=1:21
        param1(4)=5*(i-1)/100;
        Mdelta(i,1)=param1(4);
        lx=fsolve('ss_ck_c',lx01,optimset,param1);
        J=jacobian('ss_ck_c',lx,param1);
        [Mvec,Meig]=eig(J);
        Meig2=diag(Meig);
        [Meig3,Mord]=sort(Meig2);
        Mlambda=diag(Meig3);
        MM=Mvec(:,Mord);
        Mm=inv(MM);      
        kestab=-Mm(2,1)/Mm(2,2);  % stabilization constant
        
        lx01=lx;
        Mdelta(i,2)=exp(lx(1));                 % stock of capital
        Mdelta(i,3)=Mdelta(i,2)^alpha;          % otuput
        Mdelta(i,4)=exp(lx(2));                 % consumption
        Mdelta(i,5)=(n+Mdelta(i,1))*Mdelta(i,2);% investment
        Mdelta(i,6)=Mdelta(i,4)/Mdelta(i,3);    % consumption/output
        Mdelta(i,7)=Mdelta(i,5)/Mdelta(i,3);    % investment/output
        Mdelta(i,8)=(Mdelta(i,4)^(1-sigma)-1)/(1-sigma); % utility
        Mdelta(i,9)=abs(Mlambda(1,1))*100;      % speed of convergence
        Mdelta(i,10)=log(0.5)/Mlambda(1,1); 		% time to cover half the distance to steady-state
    end
    param1=param;
    lx01=lx0;
    
    % change in intertemporal elasticity of consumption
    Msigma=zeros(19,10);
    for i=1:19
        param1(5)=1.0001+(i-1)/2;
        Msigma(i,1)=param1(5);
        lx=fsolve('ss_ck_c',lx01,optimset,param1);
        J=jacobian('ss_ck_c',lx,param1);
        [Mvec,Meig]=eig(J);
        Meig2=diag(Meig);
        [Meig3,Mord]=sort(Meig2);
        Mlambda=diag(Meig3);
        MM=Mvec(:,Mord);
        Mm=inv(MM);      
        kestab=-Mm(2,1)/Mm(2,2);  % stabilization constant
        
        lx01=lx;
        Msigma(i,2)=exp(lx(1));                 % stock of capital
        Msigma(i,3)=Msigma(i,2)^alpha;          % otuput
        Msigma(i,4)=exp(lx(2));                 % consumption
        Msigma(i,5)=(n+delta)*Msigma(i,2);      % investment
        Msigma(i,6)=Msigma(i,4)/Msigma(i,3);    % consumption/output
        Msigma(i,7)=Msigma(i,5)/Msigma(i,3);    % investment/output
        Msigma(i,8)=(Msigma(i,4)^(1-Msigma(i,1))-1)/(1-Msigma(i,1)); % utility
        Msigma(i,9)=abs(Mlambda(1,1))*100;   	% speed of convergence
        Msigma(i,10)=log(0.5)/Mlambda(1,1); 		% time to cover half the distance to steady-state
    end
    
    
    % Graph representations for steady-state sensitivity to changes 
    % in a structural parameter
    
figure;
hold on

subplot(2,2,1)
[AX,H1,H2]=plotyy(Mtheta(:,1),Mtheta(:,2),Mtheta(:,1),Mtheta(:,3),'plot');
set(get(AX(1),'Ylabel'),'String','capital');
set(get(AX(2),'Ylabel'),'String','output');
title('capital stock and output');
xlabel('theta');
legend([H1,H2],'capital','output');
Axis('tight');

subplot(2,2,2)
[AX,H1,H2]=plotyy(Mtheta(:,1),Mtheta(:,4),Mtheta(:,1),Mtheta(:,8),'plot');
set(get(AX(1),'Ylabel'),'String','consumption');
set(get(AX(2),'Ylabel'),'String','utility');
title('consumption and single-period utility');
xlabel('theta');
legend([H1,H2],'consumption','utility');
Axis('tight')

subplot(2,2,3)
[AX,H1,H2]=plotyy(Mtheta(:,1),Mtheta(:,5),Mtheta(:,1),Mtheta(:,7),'plot');
set(get(AX(1),'Ylabel'),'String','investment');
set(get(AX(2),'Ylabel'),'String','output share');
title('investment');
xlabel('theta');
legend([H1,H2],'investment','output share');
Axis('tight')

subplot(2,2,4)
[AX,H1,H2]=plotyy(Mtheta(:,1),Mtheta(:,9),Mtheta(:,1),Mtheta(:,10),'plot');
set(get(AX(1),'Ylabel'),'String','speed %');
set(get(AX(2),'Ylabel'),'String','time to cover half the distance');
title('speed of convergence and discount rate');
xlabel('theta');
legend([H1,H2],'speed','time');
Axis('tight')

hold off
        


figure;
hold on

subplot(2,2,1)
[AX,H1,H2]=plotyy(Malpha(:,1),Malpha(:,2),Malpha(:,1),Malpha(:,3),'plot');
set(get(AX(1),'Ylabel'),'String','capital');
set(get(AX(2),'Ylabel'),'String','output');
title('capital stock and output');
xlabel('alpha');
legend([H1,H2],'capital','output');
Axis('tight');

subplot(2,2,2)
[AX,H1,H2]=plotyy(Malpha(:,1),Malpha(:,4),Malpha(:,1),Malpha(:,8),'plot');
set(get(AX(1),'Ylabel'),'String','consumption');
set(get(AX(2),'Ylabel'),'String','utility');
title('consumption and single-period utility');
xlabel('alpha');
legend([H1,H2],'consumption','utility');
Axis('tight')

subplot(2,2,3)
[AX,H1,H2]=plotyy(Malpha(:,1),Malpha(:,5),Malpha(:,1),Malpha(:,7),'plot');
set(get(AX(1),'Ylabel'),'String','investment');
set(get(AX(2),'Ylabel'),'String','output share');
title('investment');
xlabel('alpha');
legend([H1,H2],'investment','output share');
Axis('tight')

subplot(2,2,4)
[AX,H1,H2]=plotyy(Malpha(:,1),Malpha(:,9),Malpha(:,1),Malpha(:,10),'plot');
set(get(AX(1),'Ylabel'),'String','speed %');
set(get(AX(2),'Ylabel'),'String','time to cover half the distance');
title('speed of convergence and elasticity of physical capital');
xlabel('alpha');
legend([H1,H2],'speed','time');
Axis('tight')

hold off


figure;
hold on

subplot(2,2,1)
[AX,H1,H2]=plotyy(Mn(:,1),Mn(:,2),Mn(:,1),Mn(:,3),'plot');
set(get(AX(1),'Ylabel'),'String','capital');
set(get(AX(2),'Ylabel'),'String','output');
title('capital stock and output');
xlabel('n');
legend([H1,H2],'capital','output');
Axis('tight');

subplot(2,2,2)
[AX,H1,H2]=plotyy(Mn(:,1),Mn(:,4),Mn(:,1),Mn(:,8),'plot');
set(get(AX(1),'Ylabel'),'String','consumption');
set(get(AX(2),'Ylabel'),'String','utility');
title('consumption and single-period utility');
xlabel('n');
legend([H1,H2],'consumption','utility');
Axis('tight')

subplot(2,2,3)
[AX,H1,H2]=plotyy(Mn(:,1),Mn(:,5),Mn(:,1),Mn(:,7),'plot');
set(get(AX(1),'Ylabel'),'String','investment');
set(get(AX(2),'Ylabel'),'String','output share');
title('investment');
xlabel('n');
legend([H1,H2],'investment','output share');
Axis('tight')

subplot(2,2,4)
[AX,H1,H2]=plotyy(Mn(:,1),Mn(:,9),Mn(:,1),Mn(:,10),'plot');
set(get(AX(1),'Ylabel'),'String','speed %');
set(get(AX(2),'Ylabel'),'String','time to cover half the distance');
title('speed of convergence and population growth');
xlabel('n');
legend([H1,H2],'speed','time');
Axis('tight')

hold off


figure;
hold on

subplot(2,2,1)
[AX,H1,H2]=plotyy(Mdelta(:,1),Mdelta(:,2),Mdelta(:,1),Mdelta(:,3),'plot');
set(get(AX(1),'Ylabel'),'String','capital');
set(get(AX(2),'Ylabel'),'String','output');
title('capital stock and output');
xlabel('delta');
legend([H1,H2],'capital','output');
Axis('tight');

subplot(2,2,2)
[AX,H1,H2]=plotyy(Mdelta(:,1),Mdelta(:,4),Mdelta(:,1),Mdelta(:,8),'plot');
set(get(AX(1),'Ylabel'),'String','consumption');
set(get(AX(2),'Ylabel'),'String','utility');
title('consumption and single-period utility');
xlabel('delta');
legend([H1,H2],'consumption','utility');
Axis('tight')

subplot(2,2,3)
[AX,H1,H2]=plotyy(Mdelta(:,1),Mdelta(:,5),Mdelta(:,1),Mdelta(:,7),'plot');
set(get(AX(1),'Ylabel'),'String','investment');
set(get(AX(2),'Ylabel'),'String','output share');
title('investment');
xlabel('delta');
legend([H1,H2],'investment','output share');
Axis('tight')

subplot(2,2,4)
[AX,H1,H2]=plotyy(Mdelta(:,1),Mdelta(:,9),Mdelta(:,1),Mdelta(:,10),'plot');
set(get(AX(1),'Ylabel'),'String','speed %');
set(get(AX(2),'Ylabel'),'String','time to cover half the distance');
title('speed of convergence and depreciation rate');
xlabel('delta');
legend([H1,H2],'speed','time');
Axis('tight')

hold off



figure;
hold on

subplot(2,2,1)
[AX,H1,H2]=plotyy(Msigma(:,1),Msigma(:,2),Msigma(:,1),Msigma(:,3),'plot');
set(get(AX(1),'Ylabel'),'String','capital');
set(get(AX(2),'Ylabel'),'String','output');
title('capital stock and output');
xlabel('sigma');
legend([H1,H2],'capital','output');


subplot(2,2,2)
[AX,H1,H2]=plotyy(Msigma(:,1),Msigma(:,4),Msigma(:,1),Msigma(:,8),'plot');
set(get(AX(1),'Ylabel'),'String','consumption');
set(get(AX(2),'Ylabel'),'String','utility');
title('consumption and single-period utility');
xlabel('sigma');
legend([H1,H2],'consumption','utility');


subplot(2,2,3)
[AX,H1,H2]=plotyy(Msigma(:,1),Msigma(:,5),Msigma(:,1),Msigma(:,7),'plot');
set(get(AX(1),'Ylabel'),'String','investment');
set(get(AX(2),'Ylabel'),'String','output share');
title('investment');
xlabel('sigma');
legend([H1,H2],'investment','output share');

subplot(2,2,4)
[AX,H1,H2]=plotyy(Msigma(:,1),Msigma(:,9),Msigma(:,1),Msigma(:,10),'plot');
set(get(AX(1),'Ylabel'),'String','speed %');
set(get(AX(2),'Ylabel'),'String','time to cover half the distance');
title('speed of convergence and inverse of intertemporal elasticity of substitution');
xlabel('sigma');
legend([H1,H2],'speed','time');

hold off
