% Impulse response functions in the continuous time version of the 
% Cass Koopmans model. This program calls function ss_ck_c.m, which 
% lays out first order conditions for the planner's problem and computes 
% the transition matrix in the log-linear approximation to the model. 
% Impulse response functions are obtained by taking an initial condition 
% on the stock of capital different form the steady-state level.
%
% The single Figure produced reports the transition from the initial state,
% with a stock of capital 10% above steady-state level, to the steady-state.
% This can be changed to start from a stock of capital below steady-state
% in the line labelled "Transition path"
%
% Numerical values for structural parameters, the initial stock of capital, 
% and the number of periods can be changed in the first panel.
%
% [Copyright: J.Ruiz]

clear

% Structural parameters (benchmark values)

theta=0.04;     % time discount factor
alpha=0.30;     % output elasticity of capital
n=0;            % population growth
delta=0.1;      % depreciation rate
sigma=2;        % relative risk aversion
eps=1/sigma;    % elasticity of substitution of consumption

nobs=100;       % periods

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% steady state

kss=(alpha/(n+delta+theta))^(1/(1-alpha));  % stock of capital
yss=kss^alpha;                              % output
css=yss-(delta+n)*kss;                      % consumption
iss=(delta+n)*kss;                          % investment
uss=(css^(1-sigma)-1)/(1-sigma);            % single period utility
riy=iss/yss;                                % invesment to output ratio
rcy=css/yss;                                % consumption to output ratio

% Transition path

k0=kss*1.10;    % initial capital stock
param=[theta alpha n delta sigma];
x0=[log(kss) log(css)]';

% Transition matrix for the log-linear approximation around steady-state

J=jacobian('ss_ck_c',x0,param);

[Mvec,Meig]=eig(J);
Meig2=diag(Meig);
[Meig3,Mord]=sort(Meig2);

  if Meig3(1)>0
     'error - no solution'
  end

  if Meig3(2)<0
     'error - multiple solutions'
  end
  
Mlambda=diag(Meig3);
MM=Mvec(:,Mord);
Mm=inv(MM);      
kestab=-Mm(2,1)/Mm(2,2);  % stabilization constant

vlk=zeros(nobs,1); vlk(1)=log(k0);
vlc=zeros(nobs,1); vlc(1)=log(css)+kestab*(log(k0)-log(kss));
vk=zeros(nobs,1); vk(1)=exp(vlk(1));
vc=zeros(nobs,1); vc(1)=exp(vlc(1)); c0=vc(1);
vy=zeros(nobs,1); vy(1)=vk(1)^alpha;
vi=zeros(nobs,1); vi(1)=vy(1)-vc(1);
vu=zeros(nobs,1); vu(1)=(vc(1)^(1-sigma)-1)/(1-sigma);

for i=1:nobs-1
  lk=log(kss)+(log(k0)-log(kss))*exp(Mlambda(1,1)*i);
  lc=log(css)+(log(c0)-log(css))*exp(Mlambda(1,1)*i);
  vlk(i+1)=lk;
  vlc(i+1)=lc;
  vk(i+1)=exp(lk);
  vc(i+1)=exp(lc);
  vy(i+1)=vk(i+1)^alpha;
  vi(i+1)=vy(i+1)-vc(i+1);
  vu(i+1)=(vc(i+1)^(1-sigma)-1)/(1-sigma);
end

    vkss=ones(nobs,1).*kss;
    vcss=ones(nobs,1).*css;
    vyss=ones(nobs,1).*yss;
    viss=ones(nobs,1).*iss;
    vuss=ones(nobs,1).*uss;

% graphs    
    
figure;
hold on

subplot(3,2,1)
plot([vk,vkss]);
title('capital stock');
xlabel('periods');


subplot(3,2,2)
plot([vc,vcss]);
title('consumption');
xlabel('periods');


subplot(3,2,3)
plot([vy,vyss]);
title('output');
xlabel('periods');


subplot(3,2,4)
plot([vi,viss]);
title('investment');
xlabel('periods');


subplot(3,2,5)
plot([vu,vuss]);
title('single-period utility');
xlabel('periods');


hold off