% This program analyzes long-run (steady-state)effects of changes in either 
% the consumption tax rate (Figure 1), the income tax rate (Figure 2)or both. 
% When a single tax is changed, revenues change with the tax rate.
% When both tax rates change, they do so wither to maintain constant revenues 
% (Figure 3),or a constant ratio of revenues to output (Figure 4). 
%
% This program calls the following functions: 
%
% i) ss_ck_d.m, which solves for the steady-state, 
% ii) ss_ck_d1.m, which solves for the steady-state keeping revenues constant 
%      (in this case, the consumption tax becomes an endogenous variable, which 
%      is solved for so that revenues remain constant for any income tax rate), 
% iii)ss_ck_d2.m, which solves for the steady-state keeping constant the ratio 
%      of revenues to total income (in this case, the consumption tax is an 
%      endogenous variable which is solved for so that the revenues to output 
%      ratio is the same for any income tax rate).
%
% [Copyright: J.Ruiz]

clear

%  structural parameters

A=1.0;             	%   Technology level
beta= 0.90;      	%   Discout factor
sigma=0.5;      	%   Relative risk aversion
n=0.0;            	%   population rate
delta=0.07;        	%   depreciation rate
alpha=0.33;     	%   capital share
tauc=0.10;         	%   consumption tax rate
tauy=0.20;      	%   income tax rate


% steady state

kss0=(alpha*A*(1-tauy)/(((1+n)/beta)-(1-delta)))^(1/(1-alpha)); % capital stock
yss0=A*kss0^alpha; % output
css0=(kss0/(1+tauc))*((1-tauy)*(yss0/kss0)-(n+delta)); % consumption
iss0=(n+delta)*kss0; % investment
uss0=(css0^(1-sigma)-1)/(1-sigma); % single-period utility
revss0=tauy*yss0+tauc*css0; % public revenues
revyss0=revss0/yss0; % public revenues as % output
ydss0=(1-tauy)*yss0; % disposable income

 
    
%long run effects of a permanent change in one tax rate or in both
param=[A beta sigma n delta alpha tauc tauy];
x0=[kss0 kss0 css0 css0]';
    param1=param;param1(7)=0;
    x01=x0;
    % change in income tax rate
    Mtauy=zeros(20,9);
    for i=1:20     
        param1(8)=(i-1)/20;
        Mtauy(i,1)=param1(8);
        x=fsolve('ss_ck_d',x01,optimset,param1);
        x01=x;
        Mtauy(i,2)=x(1);   % capital stock
        Mtauy(i,3)=x(3);   % consumption
        Mtauy(i,4)=A*Mtauy(i,2)^alpha;    % output
        Mtauy(i,5)=(n+delta)*Mtauy(i,2);  % investment
        Mtauy(i,6)=(Mtauy(i,3)^(1-sigma)-1)/(1-sigma);    % single period utility      
        Mtauy(i,7)=Mtauy(i,1)*Mtauy(i,4)+param1(7)*Mtauy(i,3); % public revenues
        Mtauy(i,8)=Mtauy(i,7)/Mtauy(i,4); % public revenues as % output
        Mtauy(i,9)=(1-Mtauy(i,1))*Mtauy(i,4);   % disposable income

    end
    param1=param; param1(8)=0;
    x01=x0;
    % change in consumption tax rate
    Mtauc=zeros(20,9);
    for i=1:20     
        param1(7)=(i-1)/20;
        Mtauc(i,1)=param1(7);
        x=fsolve('ss_ck_d',x01,optimset,param1);
        x01=x;
        Mtauc(i,2)=x(1);   % capital stock
        Mtauc(i,3)=x(3);   % consumption
        Mtauc(i,4)=A*Mtauc(i,2)^alpha;    % output
        Mtauc(i,5)=(n+delta)*Mtauc(i,2);  % investment
        Mtauc(i,6)=(Mtauc(i,3)^(1-sigma)-1)/(1-sigma);    % single period utility      
        Mtauc(i,7)=param1(8)*Mtauc(i,4)+Mtauc(i,1)*Mtauc(i,3); % public revenues
        Mtauc(i,8)=Mtauc(i,7)/Mtauc(i,4); % public revenues as % output
        Mtauc(i,9)=(1-param1(8))*Mtauc(i,4);   % disposable income
    end

    
    param1=[A beta sigma n delta alpha tauy revss0];
    x01=[kss0 kss0 css0 css0 tauc]';

    % change in both tax rates, keeping tax revenues constant
    Mtauyc1=zeros(19,10);
    for i=1:19     
        param1(7)=(i-1)/20;
        Mtauyc1(i,1)=param1(7);
        x=fsolve('ss_ck_d1',x01,optimset,param1);
        x01=x;
        Mtauyc1(i,2)=x(1);   % capital stock
        Mtauyc1(i,3)=x(3);   % consumption
        Mtauyc1(i,10)=x(5);  % consumption tax rate such that the public revenues hold
        Mtauyc1(i,4)=A*Mtauyc1(i,2)^alpha;    % output
        Mtauyc1(i,5)=(n+delta)*Mtauyc1(i,2);  % investment
        Mtauyc1(i,6)=(Mtauyc1(i,3)^(1-sigma)-1)/(1-sigma);    % single period utility      
        Mtauyc1(i,7)=Mtauyc1(i,1)*Mtauyc1(i,4)+Mtauyc1(i,10)*Mtauyc1(i,3); % public revenues
        Mtauyc1(i,8)=Mtauyc1(i,7)/Mtauyc1(i,4); % public revenues as % output
        Mtauyc1(i,9)=(1-Mtauyc1(i,1))*Mtauyc1(i,4);   % disposable income
    end
    
    param1=[A beta sigma n delta alpha tauy revyss0];
    x01=[kss0 kss0 css0 css0 tauc]';

    % change in both tax rates, keeping tax revenues as % output constant
    Mtauyc2=zeros(19,10);
    for i=1:19     
        param1(7)=(i-1)/20;
        Mtauyc2(i,1)=param1(7);
        x=fsolve('ss_ck_d2',x01,optimset,param1);
        x01=x;
        Mtauyc2(i,2)=x(1);   % capital stock
        Mtauyc2(i,3)=x(3);   % consumption
        Mtauyc2(i,10)=x(5);  % consumption tax rate such that the public revenues hold
        Mtauyc2(i,4)=A*Mtauyc2(i,2)^alpha;    % output
        Mtauyc2(i,5)=(n+delta)*Mtauyc2(i,2);  % investment
        Mtauyc2(i,6)=(Mtauyc2(i,3)^(1-sigma)-1)/(1-sigma);    % single period utility      
        Mtauyc2(i,7)=Mtauyc2(i,1)*Mtauyc2(i,4)+Mtauyc2(i,10)*Mtauyc2(i,3); % public revenues
        Mtauyc2(i,8)=Mtauyc2(i,7)/Mtauyc2(i,4); % public revenues as % output
        Mtauyc2(i,9)=(1-Mtauyc2(i,1))*Mtauyc2(i,4);   % disposable income
    end
    
figure;
hold on

subplot(3,2,1);
plot(Mtauy(:,1),Mtauy(:,2));
title('effects on capital stock');
xlabel('income tax rate');
ylabel('kss');


subplot(3,2,2);
plot(Mtauy(:,1),Mtauy(:,3));
title('effects on consumption');
xlabel('income tax rate');
ylabel('css');


subplot(3,2,3);
plot(Mtauy(:,1),Mtauy(:,4));
title('effects on output');
xlabel('income tax rate');
ylabel('yss');


subplot(3,2,4);
plot(Mtauy(:,1),Mtauy(:,6));
title('effects on single-period utility');
xlabel('income tax rate');
ylabel('Uss');


subplot(3,2,5);
plot(Mtauy(:,1),Mtauy(:,9));
title('effects on disposable income');
xlabel('income tax rate');
ylabel('ydss');


subplot(3,2,6);
plot(Mtauy(:,1),Mtauy(:,7));
title('effects on public revenues');
xlabel('income tax rate');
ylabel('revss');


hold off



figure;
hold on

subplot(3,2,1);
plot(Mtauc(:,1),Mtauc(:,2));
title('effects on capital stock');
xlabel('consumption tax rate');
ylabel('kss');


subplot(3,2,2);
plot(Mtauc(:,1),Mtauc(:,3));
title('effects on consumption');
xlabel('consumption tax rate');
ylabel('css');


subplot(3,2,3);
plot(Mtauc(:,1),Mtauc(:,4));
title('effects on output');
xlabel('consumption tax rate');
ylabel('yss');


subplot(3,2,4);
plot(Mtauc(:,1),Mtauc(:,6));
title('effects on single-period utility');
xlabel('consumption tax rate');
ylabel('Uss');


subplot(3,2,5);
plot(Mtauc(:,1),Mtauc(:,9));
title('effects on disposable income');
xlabel('consumption tax rate');
ylabel('ydss');


subplot(3,2,6);
plot(Mtauc(:,1),Mtauc(:,7));
title('effects on public revenues');
xlabel('consumption tax rate');
ylabel('revss');


hold off

figure;
hold on

subplot(3,2,1);
plot(Mtauyc1(:,1),Mtauyc1(:,2));
title('effects on capital stock');
xlabel('income tax rate (keeping public revenues constant)');
ylabel('kss');


subplot(3,2,2);
plot(Mtauyc1(:,1),Mtauyc1(:,3));
title('effects on consumption');
xlabel('income tax rate (keeping public revenues constant)');
ylabel('css');


subplot(3,2,3);
plot(Mtauyc1(:,1),Mtauyc1(:,4));
title('effects on output');
xlabel('income tax rate (keeping public revenues constant)');
ylabel('yss');


subplot(3,2,4);
plot(Mtauyc1(:,1),Mtauyc1(:,10));
title('effects on consmption tax rate');
xlabel('income tax rate (keeping public revenues constant)');
ylabel('tauc');


subplot(3,2,5);
plot(Mtauyc1(:,1),Mtauyc1(:,9));
title('effects on disposable income');
xlabel('income tax rate (keeping public revenues constant)');
ylabel('ydss');


subplot(3,2,6);
plot(Mtauyc1(:,1),Mtauyc1(:,8));
title('effects on public revenues as % output');
xlabel('income tax rate (keeping public revenues constant)');
ylabel('revss/output');


hold off

figure;
hold on

subplot(3,2,1);
plot(Mtauyc2(:,1),Mtauyc2(:,2));
title('effects on capital stock');
xlabel('income tax rate (keeping public revenues to output ratio constant)');
ylabel('kss');


subplot(3,2,2);
plot(Mtauyc2(:,1),Mtauyc2(:,3));
title('effects on consumption');
xlabel('income tax rate (keeping public revenues to output ratio constant)');
ylabel('css');


subplot(3,2,3);
plot(Mtauyc2(:,1),Mtauyc2(:,4));
title('effects on output');
xlabel('income tax rate (keeping public revenues to output ratio constant)');
ylabel('yss');


subplot(3,2,4);
plot(Mtauyc2(:,1),Mtauyc2(:,10));
title('effects on consmption tax rate');
xlabel('income tax rate (keeping public revenues to output ratio constant)');
ylabel('tauc');


subplot(3,2,5);
plot(Mtauyc2(:,1),Mtauyc2(:,9));
title('effects on disposable income');
xlabel('income tax rate (keeping public revenues to output ratio constant)');
ylabel('ydss');


subplot(3,2,6);
plot(Mtauyc2(:,1),Mtauyc2(:,7));
title('effects on public revenues');
xlabel('income tax rate (keeping public revenues to output ratio constant)');
ylabel('revss');


hold off
