% This program analyzes short- and long-run effects of changes in either 
% the consumption tax rate (Figure 1), the income tax rate (Figure 2), 
% or both taxes, either keeping constant total revenues (Figure 3), or 
% the ratio of revenues to income (Figure 4). 
%
% The program calls the following functions: 
%
% i) ss_ck_d.m, which computes the steady-state as well as the transition 
%         matrix for the linear approximation to the model, 
% ii) ss_ck_d1.m, which solves for the steady-state when maintaining revenues 
%         constant (in this case, the consumption tax becomes an endogenous 
%         variable, which is solved for so that revenues remain constant for 
%         any income tax rate), and 
% iii) ss_ck_d2.m, which solves for the steady-state keeping constant the 
%         ratio of revenues to total income (in this case, the consumption tax 
%         is an endogenous variable which is solved for so that the revenues to 
%         output ratio is the same for any income tax rate).
%
% [Copyright: J.Ruiz]

clear

%  structural parameters

A=1.0;             	%   Technology level
beta= 0.90;      	%   Discout factor
sigma=0.5;      	%   Relative risk aversion
n=0.0;            	%   population rate
delta=0.07;        	%   depreciation rate
alpha=0.33;     	%   capital share
tauc=0.10;         	%   consumption tax rate
tauy=0.20;      	%   income tax rate


% steady state before the change in taxes

kss0=(alpha*A*(1-tauy)/(((1+n)/beta)-(1-delta)))^(1/(1-alpha)); % capital stock
yss0=A*kss0^alpha; % output
css0=(kss0/(1+tauc))*((1-tauy)*(yss0/kss0)-(n+delta)); % consumption
iss0=(n+delta)*kss0; % investment
uss0=(css0^(1-sigma)-1)/(1-sigma); % single-period utility
revss0=tauy*yss0+tauc*css0; % public revenues
revyss0=revss0/yss0; % public revenues as % output
ydss0=(1-tauy)*yss0; % disposable income


    
%short run and long run effects of a permanent change in one tax rate or in both
nobs=50;

param=[A beta sigma n delta alpha tauc tauy];
x0=[kss0 kss0 css0 css0]';
    param1=param;
    x01=x0;
    
    % change in income tax rate
    param1(8)=param(8)+0.01;
    x=fsolve('ss_ck_d',x01,optimset,param1);
    kss=x(1);   % capital stock
    css=x(3);   % consumption
    yss=A*kss^alpha;    % output
    iss=(n+delta)*kss;  % investment
    uss=(css^(1-sigma)-1)/(1-sigma);    % single period utility      
    revss=param1(8)*yss+param1(7)*css; % public revenues
    revyss=revss/yss; % public revenues as % output
    ydss=(1-param1(8))*yss;   % disposable income

    J=jacobian('ss_ck_d',x,param1); % transition matrix
    MA=[J(1,1) J(1,3);
        J(2,1) J(2,3)];
    MB=[J(1,2) J(1,4);
        J(2,2) J(2,4)];
    MG=-inv(MA)*MB;
    [Mvec,Meig]=eig(MG);
    Meig2=diag(Meig);
    [Meig3,Mord]=sort(abs(Meig2));
    Meig4=Meig2(Mord);
    Mlambda=diag(Meig4);
    MM=Mvec(:,Mord);
    Mm=inv(MM);
    stab=-Mm(2,1)/Mm(2,2); % stabilizing constant
    
    k0=kss0;
    c0=css+stab*(k0-kss);
    vk1=[];vk1=[vk1;k0];
    vc1=[];vc1=[vc1;c0];
    for i=1:nobs
        k1=(1/(1+n))*((1-param1(8))*A*(k0^alpha)+(1-delta)*k0-(1+param1(7))*c0);
        c1=css+stab*(k1-kss);
        c0=c1;
        k0=k1;
        vk1=[vk1;k0];  % capital stock
        vc1=[vc1;c0];  % consumption
    end
    
    vy1=A.*(vk1.^alpha);    % output
    vi1=(1+n).*vk1(2:nobs)-(1-delta).*vk1(1:nobs-1);  % investment
    vu1=(vc1.^(1-sigma)-1)./(1-sigma);    % single period utility      
    vrev1=param1(8).*vy1+param1(7).*vc1; % public revenues
    vrevy1=vrev1./vy1; % public revenues as % output
    vyd1=(1-param1(8)).*vy1;   % disposable income

    figure;
    hold on
    
    subplot(4,2,1);
    plot(vk1);
    title('Response of capital stock to an increase in income tax rate');
    xlabel('time');
    ylabel('capital stock');
    
    subplot(4,2,2);
    plot(vc1);
    title('Response of consumption to an increase in income tax rate');
    xlabel('time');
    ylabel('consumption');
    
    subplot(4,2,3);
    plot(vy1);
    title('Response of output to an increase in income tax rate');
    xlabel('time');
    ylabel('output');
    
    subplot(4,2,4);
    plot(vu1);
    title('Response of utility level to an increase in income tax rate');
    xlabel('time');
    ylabel('utility');
    
    subplot(4,2,5);
    plot(vrev1);
    title('Response of public revenue to an increase in income tax rate');
    xlabel('time');
    ylabel('public revenue');
    
    subplot(4,2,6);
    plot(vrevy1);
    title('Response of public revenue/output ratio to an increase in income tax rate');
    xlabel('time');
    ylabel('public revenue as % output');
    
    subplot(4,2,7);
    plot(vyd1);
    title('Response of disposable income to an increase in income tax rate');
    xlabel('time');
    ylabel('disposable income');
    
    hold off
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    param1=param;
    x01=x0;
    
    % change in consumption tax rate
    param1(7)=param(7)+0.01;
    x=fsolve('ss_ck_d',x01,optimset,param1);
    kss=x(1);   % capital stock
    css=x(3);   % consumption
    yss=A*kss^alpha;    % output
    iss=(n+delta)*kss;  % investment
    uss=(css^(1-sigma)-1)/(1-sigma);    % single period utility      
    revss=param1(8)*yss+param1(7)*css; % public revenues
    revyss=revss/yss; % public revenues as % output
    ydss=(1-param1(8))*yss;   % disposable income

    J=jacobian('ss_ck_d',x,param1); % transition matrix
    MA=[J(1,1) J(1,3);
        J(2,1) J(2,3)];
    MB=[J(1,2) J(1,4);
        J(2,2) J(2,4)];
    MG=-inv(MA)*MB;
    [Mvec,Meig]=eig(MG);
    Meig2=diag(Meig);
    [Meig3,Mord]=sort(abs(Meig2));
    Meig4=Meig2(Mord);
    Mlambda=diag(Meig4);
    MM=Mvec(:,Mord);
    Mm=inv(MM);
    stab=-Mm(2,1)/Mm(2,2); % stabilizing constant
    
    k0=kss0;
    c0=css+stab*(k0-kss);
    vk2=[];vk2=[vk2;k0];
    vc2=[];vc2=[vc2;c0];
    for i=1:nobs
        k1=(1/(1+n))*((1-param1(8))*A*(k0^alpha)+(1-delta)*k0-(1+param1(7))*c0);
        c1=css+stab*(k1-kss);
        c0=c1;
        k0=k1;
        vk2=[vk2;k0];  % capital stock
        vc2=[vc2;c0];  % consumption
    end
    
    vy2=A.*(vk2.^alpha);    % output
    vi2=(1+n).*vk2(2:nobs)-(1-delta).*vk2(1:nobs-1);  % investment
    vu2=(vc2.^(1-sigma)-1)./(1-sigma);    % single period utility      
    vrev2=param1(8).*vy2+param1(7).*vc2; % public revenues
    vrevy2=vrev2./vy2; % public revenues as % output
    vyd2=(1-param1(8)).*vy2;   % disposable income

    figure;
    hold on
    
    subplot(4,2,1);
    plot(vk2);
    title('Response of capital stock to an increase in consumption tax rate');
    xlabel('time');
    ylabel('capital stock');
    
    subplot(4,2,2);
    plot(vc2);
    title('Response of consumption to an increase in consumption tax rate');
    xlabel('time');
    ylabel('consumption');
    
    subplot(4,2,3);
    plot(vy2);
    title('Response of output to an increase in consumption tax rate');
    xlabel('time');
    ylabel('output');
    
    subplot(4,2,4);
    plot(vu2);
    title('Response of utility level to an increase in consumption tax rate');
    xlabel('time');
    ylabel('utility');
    
    subplot(4,2,5);
    plot(vrev2);
    title('Response of public revenue to an increase in consumption tax rate');
    xlabel('time');
    ylabel('public revenue');
    
    subplot(4,2,6);
    plot(vrevy2);
    title('Response of public revenue/output ratio to an increase in consumption tax rate');
    xlabel('time');
    ylabel('public revenue as % output');
    
    subplot(4,2,7);
    plot(vyd2);
    title('Response of disposable income to an increase in consumption tax rate');
    xlabel('time');
    ylabel('disposable income');
    
    hold off
    
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    param2=[A beta sigma n delta alpha tauy revss0];
    x02=[kss0 kss0 css0 css0 tauc]';
    
    param1=param;
    x01=x0;
            
    % change in both tax rates keeping public revenue constant
    param1(8)=param(8)+0.01; param2(7)=param(8)+0.01;
    x2=fsolve('ss_ck_d1',x02,optimset,param2);
    param1(7)=x2(5); % consumption tax rate
    kss=x2(1);   % capital stock
    css=x2(3);   % consumption
    x=[kss kss css css]';
    yss=A*kss^alpha;    % output
    iss=(n+delta)*kss;  % investment
    uss=(css^(1-sigma)-1)/(1-sigma);    % single period utility      
    revss=param1(8)*yss+param1(7)*css; % public revenues
    revyss=revss/yss; % public revenues as % output
    ydss=(1-param1(8))*yss;   % disposable income

    J=jacobian('ss_ck_d',x,param1); % transition matrix
    MA=[J(1,1) J(1,3);
        J(2,1) J(2,3)];
    MB=[J(1,2) J(1,4);
        J(2,2) J(2,4)];
    MG=-inv(MA)*MB;
    [Mvec,Meig]=eig(MG);
    Meig2=diag(Meig);
    [Meig3,Mord]=sort(abs(Meig2));
    Meig4=Meig2(Mord);
    Mlambda=diag(Meig4);
    MM=Mvec(:,Mord);
    Mm=inv(MM);
    stab=-Mm(2,1)/Mm(2,2); % stabilizing constant
    
    k0=kss0;
    c0=css+stab*(k0-kss);
    vk3=[];vk3=[vk3;k0];
    vc3=[];vc3=[vc3;c0];
    for i=1:nobs
        k1=(1/(1+n))*((1-param1(8))*A*(k0^alpha)+(1-delta)*k0-(1+param1(7))*c0);
        c1=css+stab*(k1-kss);
        c0=c1;
        k0=k1;
        vk3=[vk3;k0];  % capital stock
        vc3=[vc3;c0];  % consumption
    end
    
    vy3=A.*(vk3.^alpha);    % output
    vi3=(1+n).*vk3(2:nobs)-(1-delta).*vk3(1:nobs-1);  % investment
    vu3=(vc3.^(1-sigma)-1)./(1-sigma);    % single period utility      
    vrev3=param1(8).*vy3+param1(7).*vc3; % public revenues
    vrevy3=vrev3./vy3; % public revenues as % output
    vyd3=(1-param1(8)).*vy3;   % disposable income

    figure;
    hold on
    
    subplot(4,2,1);
    plot(vk3);
    title('Response of capital stock to an increase in both taxes keeping revenues constant');
    xlabel('time');
    ylabel('capital stock');
    
    subplot(4,2,2);
    plot(vc3);
    title('Response of consumption to an increase in both taxes keeping revenues constant');
    xlabel('time');
    ylabel('consumption');
    
    subplot(4,2,3);
    plot(vy3);
    title('Response of output to an increase in both taxes keeping revenues constant');
    xlabel('time');
    ylabel('output');
    
    subplot(4,2,4);
    plot(vu3);
    title('Response of utility level to an increase in both taxes keeping revenues constant');
    xlabel('time');
    ylabel('utility');
    
    subplot(4,2,5);
    plot(vrev3);
    title('Response of public revenue to an increase in both taxes keeping revenues constant');
    xlabel('time');
    ylabel('public revenue');
    
    subplot(4,2,6);
    plot(vrevy3);
    title('Response of public revenue/output ratio to an increase in both taxes');
    xlabel('time');
    ylabel('public revenue as % output');
    
    subplot(4,2,7);
    plot(vyd3);
    title('Response of disposable income to an increase in both taxes');
    xlabel('time');
    ylabel('disposable income');
    
    hold off
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    param2=[A beta sigma n delta alpha tauy revyss0];
    x02=[kss0 kss0 css0 css0 tauc]';
    
    param1=param;
    x01=x0;
            
    % change in both tax rates keeping public revenue constant
    param1(8)=param(8)+0.01; param2(7)=param(8)+0.01;
    x2=fsolve('ss_ck_d2',x02,optimset,param2);
    param1(7)=x2(5); % consumption tax rate
    kss=x2(1);   % capital stock
    css=x2(3);   % consumption
    x=[kss kss css css]';
    yss=A*kss^alpha;    % output
    iss=(n+delta)*kss;  % investment
    uss=(css^(1-sigma)-1)/(1-sigma);    % single period utility      
    revss=param1(8)*yss+param1(7)*css; % public revenues
    revyss=revss/yss; % public revenues as % output
    ydss=(1-param1(8))*yss;   % disposable income

    J=jacobian('ss_ck_d',x,param1); % transition matrix
    MA=[J(1,1) J(1,3);
        J(2,1) J(2,3)];
    MB=[J(1,2) J(1,4);
        J(2,2) J(2,4)];
    MG=-inv(MA)*MB;
    [Mvec,Meig]=eig(MG);
    Meig2=diag(Meig);
    [Meig3,Mord]=sort(abs(Meig2));
    Meig4=Meig2(Mord);
    Mlambda=diag(Meig4);
    MM=Mvec(:,Mord);
    Mm=inv(MM);
    stab=-Mm(2,1)/Mm(2,2); % stabilizing constant
    
    k0=kss0;
    c0=css+stab*(k0-kss);
    vk4=[];vk4=[vk4;k0];
    vc4=[];vc4=[vc4;c0];
    for i=1:nobs
        k1=(1/(1+n))*((1-param1(8))*A*(k0^alpha)+(1-delta)*k0-(1+param1(7))*c0);
        c1=css+stab*(k1-kss);
        c0=c1;
        k0=k1;
        vk4=[vk4;k0];  % capital stock
        vc4=[vc4;c0];  % consumption
    end
    
    vy4=A.*(vk4.^alpha);    % output
    vi4=(1+n).*vk4(2:nobs)-(1-delta).*vk4(1:nobs-1);  % investment
    vu4=(vc4.^(1-sigma)-1)./(1-sigma);    % single period utility      
    vrev4=param1(8).*vy4+param1(7).*vc4; % public revenues
    vrevy4=vrev4./vy4; % public revenues as % output
    vyd4=(1-param1(8)).*vy4;   % disposable income

    figure;
    hold on
    
    subplot(4,2,1);
    plot(vk4);
    title('Response of capital stock to an increase in both taxes keeping revenues/output ratio constant');
    xlabel('time');
    ylabel('capital stock');
    
    subplot(4,2,2);
    plot(vc4);
    title('Response of consumption to an increase in both taxes');
    xlabel('time');
    ylabel('consumption');
    
    subplot(4,2,3);
    plot(vy4);
    title('Response of output to an increase in both taxes');
    xlabel('time');
    ylabel('output');
    
    subplot(4,2,4);
    plot(vu4);
    title('Response of utility level to an increase in both taxes');
    xlabel('time');
    ylabel('utility');
    
    subplot(4,2,5);
    plot(vrev4);
    title('Response of public revenue to an increase in both taxes');
    xlabel('time');
    ylabel('public revenue');
    
    subplot(4,2,6);
    plot(vrevy4);
    title('Response of public revenue/output ratio to an increase in both taxes');
    xlabel('time');
    ylabel('public revenue as % output');
    
    subplot(4,2,7);
    plot(vyd4);
    title('Response of disposable income to an increase in both taxes');
    xlabel('time');
    ylabel('disposable income');
    
    hold off
    
