% This program computes the transition in the discrete time version of the Cass-Koopmans 
% model without uncertainty using different approaches, depending on whether the stability 
% condition is used every period or just at the initial period. This program calls function 
% ss_ck_d.m, which lays out first order conditions for the discrete-time version of the 
% Cass-Koopmans economy. These conditions can be used either to compute steady-state levels 
% of the endogenous variables, as well as the transition matrix for the linear approximation 
% to the model.
%
% [Copyright: J.Ruiz]

clear

%  structural parameters

A=1.0;             	%   Technology level
beta= 0.90;      	   %   Discout factor
sigma=0.5;      	   %   Relative risk aversion
n=0.0;            	%   Population rate
delta=0.07;        	%   Depreciation rate
alpha=0.33;     	   %   Capital share
tauc=0.2;          	%   Consumption tax rate
tauy=0.12478;      	%   Income tax rate


% steady state

kss0=(alpha*A*(1-tauy)/(((1+n)/beta)-(1-delta)))^(1/(1-alpha)); % capital stock
yss0=A*kss0^alpha; % output
css0=(kss0/(1+tauc))*((1-tauy)*(yss0/kss0)-(n+delta)); % consumption
iss0=(n+delta)*kss0; % investment
uss0=(css0^(1-sigma)-1)/(1-sigma); % single-period utility
revss0=tauy*yss0+tauc*css0; % public revenues
revyss0=revss0/yss0; % public revenues as % output
ydss0=(1-tauy)*yss0; % disposable income

    % other way to calculate the steady state
    param=[A beta sigma n delta alpha tauc tauy];
    x0=[kss0 kss0 css0 css0]';
    x=fsolve('ss_ck_d',x0,optimset,param);
    kss=x(1);
    css=x(3);
    yss=A*kss^alpha; 
    iss=(n+delta)*kss; 
    uss=(css^(1-sigma)-1)/(1-sigma);      
    revss=tauy*yss+tauc*css; 
    revyss=revss/yss; 
    ydss=(1-tauy)*yss;
    
    J=jacobian('ss_ck_d',x,param); % transition matrix
    MA=[J(1,1) J(1,3);
        J(2,1) J(2,3)];
    MB=[J(1,2) J(1,4);
        J(2,2) J(2,4)];
    MG=-inv(MA)*MB;
    [Mvec,Meig]=eig(MG);
    Meig2=diag(Meig);
    [Meig3,Mord]=sort(abs(Meig2));
    
      if Meig3(1)>1
        'error - no solution'
      end

      if Meig3(2)<1
        'error - multiple solutions'
      end
    
    Meig4=Meig2(Mord);
    Mlambda=diag(Meig4);
    MM=Mvec(:,Mord);
    Mm=inv(MM);
    
    stab=-Mm(2,1)/Mm(2,2); % stabilizing constant

% We choose an initial condition for the stock of capital
    
    k00=kss*0.99; 
    
% solution using the model (imposing stability just at the initial period)
    nobs=50;
    k0=k00; 
    c0=css+stab*(k0-kss);
    vk1=[];vk1=[vk1;k0];
    vc1=[];vc1=[vc1;c0];
    for i=1:nobs
        k1=(1/(1+n))*((1-tauy)*A*(k0^alpha)+(1-delta)*k0-(1+tauc)*c0);
        c1=c0*((beta/(1+n))*((1-tauy)*A*alpha*(k1^(alpha-1))...
            +(1-delta)))^(1/sigma);
        c0=c1;
        k0=k1;
        vk1=[vk1;k0];
        vc1=[vc1;c0];
    end
    
% solution using the linear aproximation to the model (imposing stability just at the initial period)
    k0=k00;
    c0=css+stab*(k0-kss);
    vk2=[];vk2=[vk2;k0];
    vc2=[];vc2=[vc2;c0];
    for i=1:nobs
        k1=kss+MG(1,1)*(k0-kss)+MG(1,2)*(c0-css);
        c1=css+MG(2,1)*(k0-kss)+MG(2,2)*(c0-css);
        c0=c1;
        k0=k1;
        vk2=[vk2;k0];
        vc2=[vc2;c0];
    end

% solution using the exponential convergence (imposing stability just at the initial period)
    k0=k00;
    c0=css+stab*(k0-kss);
    vk3=[];vk3=[vk3;k0];
    vc3=[];vc3=[vc3;c0];
    for i=1:nobs
        k1=kss+(Mlambda(1,1)^i)*(k0-kss);
        c1=css+(Mlambda(1,1)^i)*(c0-css);
        vk3=[vk3;k1];
        vc3=[vc3;c1];
    end
 
% solution using the linear aproximation to the model (imposing stability at all points in time)
    k0=k00;
    c0=css+stab*(k0-kss);
    vk4=[];vk4=[vk4;k0];
    vc4=[];vc4=[vc4;c0];
    for i=1:nobs
        k1=kss+MG(1,1)*(k0-kss)+MG(1,2)*(c0-css);
        c1=css+stab*(k0-kss);
        c0=c1;
        k0=k1;
        vk4=[vk4;k0];
        vc4=[vc4;c0];
    end

% solution using the non linear model (imposing stability at all points in time)
    k0=k00;
    c0=css+stab*(k0-kss);
    vk5=[];vk5=[vk5;k0];
    vc5=[];vc5=[vc5;c0];
    for i=1:nobs
        k1=(1/(1+n))*((1-tauy)*A*(k0^alpha)+(1-delta)*k0-(1+tauc)*c0);
        c1=css+stab*(k0-kss);
        c0=c1;
        k0=k1;
        vk5=[vk5;k0];
        vc5=[vc5;c0];
    end

figure;
hold on

subplot(3,2,1)
plot(vk2);
title('solution using the linear approximation to the model (imposing stability just at the initial period)');
ylabel('capital stock 2');
xlabel('periods');
Axis('tight');

subplot(3,2,2)
plot(vk3);
title('solution using the exponential convergence (imposing stability just at the initial period)');
ylabel('capital stock 3');
xlabel('periods');
Axis('tight');

subplot(3,2,3)
plot(vk4);
title('solution using the linear approximation to the model (imposing stability at all points in time)');
ylabel('capital stock 4');
xlabel('periods');
Axis('tight');

subplot(3,2,4)
plot(vk5);
title('solution using the non linear model (imposing stability at all points in time)');
ylabel('capital stock 5');
xlabel('periods');
Axis('tight');

subplot(3,2,5)
plot(vk2-vk3);
title('capital stock 2 -capital stock 3');
xlabel('periods');
Axis('tight');

subplot(3,2,6)
plot(vk4-vk5);
title('capital stock 4 -capital stock 5');
xlabel('periods');
Axis('tight');

hold off



